package implementations;
import specifications.StackSpecification;
import java.util.Random;


/**
 * Implements {@link StackSpecification} interface using a linked list of {@link intLink}s
 * 
 * The stack is faulty (for teaching purposes). There is a percentage chance that on every
 * interaction with the stack that changes its state (i.e. the pushes and pops) that the 
 * state of one of the other elements of the stack will be corrupted (incremented by 1).
 * The element that is corrupted is randomly chosen.
 * @author J Paul Gibson
 */
public class FaultyStack  extends Stack {
	
	
    /**
     * Percentage value should be between 0 and 100 - the probability that there is a faulty
     * push or pop that corrupts the stack state. Default value is 10. This can be changed in
     * the constructor.
     */
	 private int faultPercentage =10;
	 
	 /**
	  * The random number generator used to generate random faults
	  */
	 private final Random RNG = new Random();
	 
	 /**
	  * Constructs an empty (bounded) faulty stack of integers
	  * @param sizeBound specifies the maximum size of the faulty stack to be constructed
	  * @throws IllegalArgumentException if the specified size is not a positive integer
	  */
	 public FaultyStack (int sizeBound) throws IllegalArgumentException{
	    super(sizeBound);
	 }
	 
	 /**
	  * Constructs an empty (bounded) faulty stack of integers
	  * @param sizeBound specifies the maximum size of the faulty stack to be constructed
	  * @param faultPercentage  is the probability (as a percentage) that there is a faulty
      * push or pop that corrupts the stack state.
	  * @throws IllegalArgumentException if the specified size is not a positive integer
	  */
	 public FaultyStack (int sizeBound, int faultPercentage) throws IllegalArgumentException{
	    super(sizeBound);
	    if (faultPercentage<0) this.faultPercentage=0;
	    else if (faultPercentage>100) this.faultPercentage=100;
	    else this.faultPercentage=faultPercentage;
	 }
	
@Override
/**
 * There is a probability that the push will corrupt one of the elements already in the stack.
 */
public void push (int x){
	
  super.push(x);
  
  if (RNG.nextInt(100) <faultPercentage) {
	  int randomElementToCorrupt = RNG.nextInt(numberOfElements);
	  intLink linkToCorrupt = l;
	  for (int i=0; i<randomElementToCorrupt; i++) linkToCorrupt=linkToCorrupt.tail;
	  linkToCorrupt.head++;
  }
	 
}

@Override
/**
 * There is a probability that the pop will corrupt one of the elements already in the stack.
 */
public void pop (){
   super.pop();
   
   if (RNG.nextInt(100) <faultPercentage) {
		  int randomElementToCorrupt= RNG.nextInt(numberOfElements);
		  intLink linkToCorrupt =l;
		  for (int i=0; i<randomElementToCorrupt; i++) linkToCorrupt=linkToCorrupt.tail;
		  linkToCorrupt.head++;
   }
}
}
