package implementations;
import specifications.StackSpecification;


/**
 * Implements {@link StackSpecification} interface using a linked list of {@link intLink}s
 * @author J Paul Gibson
 */
public class Stack  implements StackSpecification {
	
	/**
	 * Used in the implementation of a {@link Stack} of integers as a linked list
	 * @author J Paul Gibson
	 * @see Stack
	 *
	 */
	protected class intLink {
	int head;
	intLink tail;
	intLink(int h, intLink t){head = h;  tail = t;}
	}

/**
 *  The maximum number of elements that the Stack can contain - cannot be changed once initialised
 */
protected static int size;

/**
 * The number of elements that are currently stored on the stack
 */
protected int numberOfElements;

/**
 * The first link in the list of integer elements used to store the state of the stack
 */
protected intLink l;


/**
 * Constructs an empty (bounded) stack of integers
 * @param sizeBound specifies the maximum size of the Stack to be constructed
 * @throws IllegalArgumentException if the specified size is not a positive integer
 */
public Stack (int sizeBound) throws IllegalArgumentException{
   if (sizeBound<1) throw new IllegalArgumentException("The size of the stack must be a postive integer"); 
   else {l =null; size = sizeBound; numberOfElements=0;}
}

@Override
public int getNumberOfElements(){return numberOfElements;}

@Override
public int getSize(){return size;}

@Override
public void push (int x){
  if (numberOfElements==size) throw new IllegalStateException("Cannot push onto a stack that is full");
  else{intLink il = new intLink(x,l);
       l = il;
       numberOfElements++;}
}

@Override
public int head (){
  if (l==null) throw new IllegalStateException("Cannot read the head of a stack that is empty");
  else return l.head;
}

@Override
public void pop (){
   if (l==null) throw new IllegalStateException("Cannot pop the head of a stack that is empty");
   else{ l = l.tail; 
         numberOfElements--;}
}

/**
 * Generate a string representation of the stack of the form:<br>
 * head -> 3-> 2-> 1-> .
 */
public String toString(){
  String temp  ="Stack: head -> ";
  intLink il =l;
  while (il!=null){temp = temp+ il.head+ "-> "; 
                   il = il.tail;}
  temp = temp + ".";
  return temp;
}


/**
 * Check that:
 * <ul>
 * <li> the size is a positive integer </li>, and
 * <li> the numberOfElements is bounded by the size</li>, and
 * <li> the numberOfElements is non-negative</li>, and
 * <li> an empty stack has size 0 </li>
 *  </ul>
 */
public boolean invariant(){
	return (getSize()>0 && 
			getNumberOfElements()<= getSize() && 
			getNumberOfElements() >=0 &&
			!(is_empty() && getNumberOfElements() !=0));
}

/**
 * @return if the stack is empty
 */
public boolean is_empty(){return (getNumberOfElements()==0);}

/**
 * @return if the stack is full
 */
public boolean is_full(){return (getSize() == getNumberOfElements());}

}


