package specifications;


/**
 * 
 * @author J Paul Gibson
 * 
 * A FIFO bounded queue of integers with a maximum number of elements that must be at least 1.
 *
 */
public interface QueueSpecification {

	/**
	 * 
	 * @return the maximum number of elements that the queue can hold
	 */
	public int getSize();
	
	/**
	 * 
	 * @return the number of elements that are currently stored on the queue
	 */
	public int getNumberOfElements();
	
	
	/**
	 * 
	 * @return  if the queue is full
	 */
	public boolean is_full();
	
	/**
	 * 
	 * @return if the queue is empty
	 */
	public boolean is_empty();
	
	/**
	 * 
	 * @param x is value being pushed onto queue
	 * @throws IllegalStateException if we try to push onto a queue that is full
	 */
	public void push (int x) throws IllegalStateException;
	
	
	/**
	 * 
	 * @return  head of queue without changing state,
	 * where the head is the element that has been in the queue for the longest time
	 * @throws IllegalStateException if we try to read the head of an empty queue
	 */
	public int head () throws IllegalStateException;
	
	/**
	 *  remove head of queue if it is not empty,
	 *  where the head is the element that has been in the queue for the longest time
	 *  @throws IllegalStateException if we try to pop from an empty QueueFromStacks
	 */
	public void pop () throws IllegalStateException;
	
	/**
	 * 
	 * @return if the queue is in a safe state and respects the specified requirements
	 */
	public boolean invariant();
	
}
