package specifications;

/**
 * A LIFO bounded stack of integers with a maximum number of elements (that must be at least 1)
 * @author J Paul Gibson
 */
public interface StackSpecification {

	/**
	 * 
	 * @return the maximum number of elements that the stack can hold
	 */
	public int getSize();
	
	/**
	 * 
	 * @return the number of elements that are currently stored on the stack
	 */
	public int getNumberOfElements();
	
	/**
	 * 
	 * @return if the stack is full
	 */
	public boolean is_full();
	
	/**
	 * 
	 * @return if the stack is empty
	 */
	public boolean is_empty();
	
	/**
	 * 
	 * @param x is value being pushed onto stack
	 * @throws IllegalStateException if we try to push onto a stack that is full
	 */
	public void push (int x) throws IllegalStateException;
	
	
	/**
	 * 
	 * @return  head of stack without changing state, 
	 * where the head is the element that has been in the stack for the shortest time
	 * @throws IllegalStateException if we try to read the head of an empty stack
	 */
	public int head () throws IllegalStateException;
	
	/**
	 *  remove head of stack if it is not empty, 
	 *  where the head is the element that has been in the stack for the shortest time
	 *  @throws IllegalStateException if we try to pop from an empty stack
	 */
	public void pop () throws IllegalStateException;
	
	/**
	 * 
	 * @return if the stack is in a safe state
	 */
	public boolean invariant();
	
}
