package tests.unit;

import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import implementations.QueueFromStacks;
import specifications.QueueSpecification;


public class JUnit_Queue {
	
	QueueSpecification testQueue;
	
	/**
	 * Initialise Queue to be tested to have a maximum of 3 elements, and to be empty
	 * @throws Exception
	 */
	@Before
	public  void setUp() throws Exception{
		
		testQueue = new QueueFromStacks(3);
		Assert.assertTrue(testQueue.invariant());
	}
	
	@After
	public void tearDown() throws Exception {
		testQueue = null;
	}
	
	/**
	 * popping from an empty queue should throw an exception
	 */
	@Test (expected = IllegalStateException.class)
	public void testpopFromEmpty(){
		Assert.assertTrue(testQueue.is_empty());
		testQueue.pop();
		Assert.assertTrue(testQueue.invariant());
		
	}
	
	/**
	 * pushing to a full queue should throw an exception
	 */
	@Test (expected = IllegalStateException.class)
	public void testpushToFull(){
		testQueue.push(1);
		testQueue.push(2);
		testQueue.push(3);
		Assert.assertTrue(testQueue.is_full());
		testQueue.push(4); 
		Assert.assertTrue(testQueue.invariant());
	}
	
	/**
	 * the head of a queue with 1 element was the last element pushed on
	 */
	@Test
	public void testFIFO1(){
		testQueue.push(10);
		Assert.assertEquals(1,testQueue.getNumberOfElements());
		Assert.assertEquals(10,testQueue.head());
		Assert.assertTrue(testQueue.invariant());
	}
	
	/**
	 * the head of a queue with 2 elements was the first element pushed on
	 */
	@Test
	public void testFIFO12(){
		testQueue.push(10);
		testQueue.push(20);
		Assert.assertEquals(10,testQueue.head());
		Assert.assertTrue(testQueue.invariant());
	}

	/**
	 * the head of a full queue was the first element pushed on
	 */
	@Test
	public void testFIFO13(){
		testQueue.push(10);
		testQueue.push(20);
		testQueue.push(30);
		Assert.assertEquals(10,testQueue.head());
		Assert.assertTrue(testQueue.invariant());
	}
	
	/**
	 * Popping from full to empty
	 */
	@Test
	public void testpop(){
		testQueue.push(10);
		testQueue.push(20);
		testQueue.push(30);
		Assert.assertTrue(testQueue.is_full());
		Assert.assertEquals(10,testQueue.head());
		testQueue.pop();
		Assert.assertEquals(20,testQueue.head());
		testQueue.pop();
		Assert.assertEquals(30,testQueue.head());
		testQueue.pop();
		Assert.assertTrue(testQueue.is_empty());
		Assert.assertTrue(testQueue.invariant());
	}
}
