package tests.unit;

import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import implementations.Stack;

public class JUnit_Stack {
	
	Stack testStack;
	
	/**
	 * Initialise Stack to be tested to have a maximum of 3 elements, and to be empty
	 * @throws Exception
	 */
	@Before
	public  void setUp() throws Exception{
		
		testStack = new Stack(3);
		Assert.assertTrue(testStack.invariant());
	}
	
	@After
	public void tearDown() throws Exception {
		testStack = null;
	}
	
	/**
	 * popping from an empty stack should throw an exception
	 */
	@Test (expected = IllegalStateException.class)
	public void testpopFromEmpty(){
		Assert.assertTrue(testStack.is_empty());
		testStack.pop();
		Assert.assertTrue(testStack.invariant());
		
	}
	
	/**
	 * pushing to a full stack should throw an exception
	 */
	@Test (expected = IllegalStateException.class)
	public void testpushToFull(){
		testStack.push(1);
		testStack.push(2);
		testStack.push(3);
		Assert.assertTrue(testStack.is_full());
		testStack.push(4); 
		Assert.assertTrue(testStack.invariant());
	}
	
	/**
	 * the head of a stack with 1 element was the last element pushed on
	 */
	@Test
	public void testLIFO1(){
		testStack.push(10);
		Assert.assertEquals(1,testStack.getNumberOfElements());
		Assert.assertEquals(10,testStack.head());
		Assert.assertTrue(testStack.invariant());
	}
	
	/**
	 * the head of a stack with 2 elements was the last element pushed on
	 */
	@Test
	public void testLIFO12(){
		testStack.push(10);
		testStack.push(20);
		Assert.assertEquals(20,testStack.head());
		Assert.assertTrue(testStack.invariant());
	}

	/**
	 * the head of a full stack was the last element pushed on
	 */
	@Test
	public void testLIFO13(){
		testStack.push(10);
		testStack.push(20);
		testStack.push(30);
		Assert.assertEquals(30,testStack.head());
		Assert.assertTrue(testStack.invariant());
	}
	
	/**
	 * Popping from full to empty
	 */
	@Test
	public void testPopFullToEmpty(){
		testStack.push(10);
		testStack.push(20);
		testStack.push(30);
		Assert.assertTrue(testStack.is_full());
		Assert.assertEquals(30,testStack.head());
		testStack.pop();
		Assert.assertEquals(20,testStack.head());
		testStack.pop();
		Assert.assertEquals(10,testStack.head());
		testStack.pop();
		Assert.assertTrue(testStack.is_empty());
		Assert.assertTrue(testStack.invariant());
	}
}
